#include <linux/ps2/dev.h>
#include <stdio.h>
#include <time.h>
#include "ps2gs.h"
#include "ps2dma.h"
#include "packet.h"
#include "texture.h"
#include "primitives.h"

#include "PS2Utils.h"

ps2_gs_gparam		*gp;											
ps2_gs_dbuff		g_db;											
ps2_gs_finish		g_finish;										

int g_fd_gs;
int g_frame=0,g_odev;	
int g_next2k;
int g_textop64;			//start of texture buffer in memory
char *g_program_name;

// these are set below
int	g_inter;
int	g_out_mode;
int	g_ff_mode;
int	g_resolution;
int	g_refresh_rate;

int	g_psm			= PS2_GS_PSMCT32;
int	g_zpsm			= PS2_GS_PSMZ24;
int	g_zbits			= 0;  //no zbuffer needed

// internal functions
int release();
int acquire();

void DisplayInit(int crt_mode)
{
	switch (crt_mode)
	{
	case PS2_GS_DTV  :
		g_inter = PS2_GS_NOINTERLACE;
		g_out_mode = PS2_GS_DTV;
		g_ff_mode = PS2_GS_FRAME;
		g_resolution = PS2_GS_480P;
		break;
	case PS2_GS_NTSC :
		g_inter = PS2_GS_NOINTERLACE;
		g_out_mode = PS2_GS_NTSC;
		g_ff_mode = PS2_GS_FRAME;
		g_resolution = PS2_GS_640x480;
		g_refresh_rate = PS2_GS_60Hz;
		break;
	case PS2_GS_PAL :
		g_inter = PS2_GS_NOINTERLACE;
		g_out_mode = PS2_GS_PAL;
		g_ff_mode = PS2_GS_FRAME;
		g_resolution = PS2_GS_640x480;
		g_refresh_rate = PS2_GS_60Hz;
		printf("PAL not supported yet\n");
		break;
	default :
	case PS2_GS_VESA :
		g_inter = PS2_GS_NOINTERLACE;
		g_out_mode = PS2_GS_VESA;
		g_ff_mode = PS2_GS_FRAME;
		g_resolution = PS2_GS_640x480;
		g_refresh_rate = PS2_GS_60Hz;
		break;
	}

	gp = ps2_gs_get_gparam();										

	g_fd_gs = ps2_gs_open(-1);		
		
	ps2_gs_vc_graphicsmode();										
		
	ps2_gs_reset(0,g_inter,g_out_mode,g_ff_mode,g_resolution,g_refresh_rate);	
	
	g_next2k = ps2_gs_set_dbuff(&g_db, g_psm, gp->width, gp->height,	
					(g_zbits == 0) ? 0 : PS2_GS_ZGREATER, g_zpsm, 1);
	
	// background color for each of the two buffers
	*(__u64 *)&g_db.clear0.rgbaq = PS2_GS_SETREG_RGBAQ				
								(0, 0, 0, 128, 0x3f800000);  
	*(__u64 *)&g_db.clear1.rgbaq = PS2_GS_SETREG_RGBAQ				
								(0, 0, 0, 128, 0x3f800000);
	
	ps2_gs_put_drawenv(&g_db.giftag1);										
	ps2_gs_set_finish(&g_finish);									
	ps2_gs_wait_finish(&g_finish);
	g_odev=!ps2_gs_sync_v(0);
	ps2_gs_start_display(1);
	ps2_gs_vc_enablevcswitch(acquire, release);
	
	g_textop64= g_next2k*2048/64;
}

void DisplayFlip()
{
	ps2_gs_vc_lock();											

	g_odev = !ps2_gs_sync_v(0);
	
	ps2_gs_set_half_offset((g_frame & 1) ? &g_db.draw1 : &g_db.draw0, g_odev);
		
	ps2_gs_swap_dbuff(&g_db, g_frame);							

	g_frame++;													

	ps2_gs_vc_unlock();		
}

void DisplayDeInit()
{
	if (g_fd_gs>=0)													
	{
		ps2_gs_close();												
		g_fd_gs = -1;												
	}	
}

void ScreenCapture(char *filename)
{
	FILE				*file;
	BitmapFile			bitmap;
	ps2_image			img;
	Dword				*image_data32,pixel32,linesize,totalsize;
	Word				xsize,ysize,*image_data16,pixel16;
	Byte				*image_line;
	
	xsize = 640;
	ysize = 480;

	if(g_psm == PS2_GS_PSMCT32)
	{
		image_data32 = (Dword *)memalign(128,xsize*ysize*4);
		ps2_gs_set_image(&img, 0, xsize/64, g_psm, 0,0, xsize, ysize, image_data32);
		ps2_gs_store_image(&img);
	}
	else if(g_psm == PS2_GS_PSMCT16)
	{
		image_data16 = (Word *)memalign(128,xsize*ysize*2);
		ps2_gs_set_image(&img, 0, xsize/64, g_psm, 0,0, xsize, ysize, image_data16);
		ps2_gs_store_image(&img);
	}
	else
	{	printf("Error capturing screen, unsupported g_psm type\n");
		return;
	}

	file = fopen(filename, "wb");

	memset( &(bitmap.bitmapfileheader), 0, 14);
	memset( &(bitmap.bitmapinfoheader), 0, sizeof(BitmapInfoHeader));

	linesize = 640*3;
	totalsize = 14 + sizeof(BitmapInfoHeader) + linesize * ysize;
	image_line = (Byte *)malloc(linesize);

	bitmap.bitmapfileheader.bfType = BITMAP_ID;
	bitmap.bitmapfileheader.bfSize = totalsize;
	bitmap.bitmapfileheader.bfReserved1 = 0;
	bitmap.bitmapfileheader.bfReserved2 = 0;
	bitmap.bitmapfileheader.bfOffBits = 54;
	
	bitmap.bitmapinfoheader.biSize = 40;
	bitmap.bitmapinfoheader.biWidth = xsize;
	bitmap.bitmapinfoheader.biHeight = ysize;
	bitmap.bitmapinfoheader.biPlanes = 1;
	bitmap.bitmapinfoheader.biBitCount = 24;
	bitmap.bitmapinfoheader.biSizeImage = linesize * ysize;
	bitmap.bitmapinfoheader.biXPelsPerMeter = 0x0ECE;
	bitmap.bitmapinfoheader.biYPelsPerMeter = 0x0ECE;

	//Again with this bitmapfileheader, do it separately
	//fwrite(&(bitmap.bitmapfileheader), 14, 1, file);  //should be this?
	fwrite(&(bitmap.bitmapfileheader.bfType),2,1,file);
	fwrite(&(bitmap.bitmapfileheader.bfSize),4,1,file);
	fwrite(&(bitmap.bitmapfileheader.bfReserved1),2,1,file);
	fwrite(&(bitmap.bitmapfileheader.bfReserved2),2,1,file);
	fwrite(&(bitmap.bitmapfileheader.bfOffBits),4,1,file);
	
	fwrite(&(bitmap.bitmapinfoheader), sizeof(BitmapInfoHeader), 1, file);
	
	fseek(file,bitmap.bitmapfileheader.bfOffBits,SEEK_SET);

	for(int y=0; y<ysize; y++)
	{
		for(int x=0; x<xsize; x++)
		{
			if(g_psm == PS2_GS_PSMCT32)
			{
				pixel32 = image_data32[(ysize-y-1)*xsize+x];
				image_line[x*3+0] = pixel32 >> 16;
				image_line[x*3+1] = pixel32 >> 8;
				image_line[x*3+2] = pixel32 >> 0;
			}
			else
			{
				pixel16 = image_data16[(ysize-y-1)*xsize+x];
				image_line[x*3+0] = (pixel16 >> 10) << 3;
				image_line[x*3+1] = (pixel16 >> 5) << 3;
				image_line[x*3+2] = pixel16 << 3;
			}
		}

		fwrite(image_line, linesize, 1, file);
	}
	
	fclose(file);

	free(image_line);
	
	if(g_psm == PS2_GS_PSMCT32)
		free(image_data32);
	else free(image_data16);
}

// internal stuff

int release()
{
	if (g_fd_gs >= 0) {
		ps2_gs_close();
		g_fd_gs = -1;
	}
	
	return PS2_GS_VC_REL_SUCCESS;
}

int acquire()
{
	g_fd_gs = ps2_gs_open(-1);
	if (g_fd_gs < 0) {
	    return PS2_GS_VC_ACQ_FAILURE;
	}
	
	ps2_gs_reset(0, g_inter, g_out_mode, g_ff_mode, g_resolution,
				 g_refresh_rate);
	
	return PS2_GS_VC_ACQ_SUCCESS;
}

