//============================================================================
//
//   SSSS    tt          lll  lll       
//  SS  SS   tt           ll   ll        
//  SS     tttttt  eeee   ll   ll   aaaa 
//   SSSS    tt   ee  ee  ll   ll      aa
//      SS   tt   eeeeee  ll   ll   aaaaa  --  "An Atari 2600 VCS Emulator"
//  SS  SS   tt   ee      ll   ll  aa  aa
//   SSSS     ttt  eeeee llll llll  aaaaa
//
// Copyright (c) 1995-1998 by Bradford W. Mott
//
// See the file "license" for information on usage and redistribution of
// this file, and for a DISCLAIMER OF ALL WARRANTIES.
//
// $Id: SndUnix.hxx,v 1.2 2002/01/08 17:11:32 stephena Exp $
//============================================================================

#ifndef SOUNDUNIX_HXX
#define SOUNDUNIX_HXX

#include "bspf.hxx"
#include "Sound.hxx"

/**
  This class implements the sound API for the PS2.
*/

class SoundPS2 : public Sound
{
  public:
    /**
      Create a new sound object
    */
    SoundPS2(int volume);
 
    /**
      Destructor
    */
    virtual ~SoundPS2();

  public: 
    /**
      Set the value of the specified sound register

      @param reg The sound register to set
      @param val The new value for the sound register
    */
    virtual void set(Sound::Register reg, uInt8 val);

    /**
      Set the mute state of the sound object

      @param state Mutes sound iff true
    */
    virtual void mute(bool state);

	void handleTime();

  private:

	// volume to reset on exit
	int originalVolume;

    // Indicates if the sound is muted or not
    bool myMute;

	int dsp_fd;
	int mixer_fd;

	int sampleRate;

	int fragmentSize;
	unsigned char* fragmentBuffer;

    // Buffers for the audio registers used so we only 
    // send "changes" to the stella-sound process
    uInt8 myAUDC0;
    uInt8 myAUDC1;
    uInt8 myAUDF0;
    uInt8 myAUDF1;
    uInt8 myAUDV0;
    uInt8 myAUDV1;

	void openMixer(int changeVolume);
	void closeMixer();

	void setupSoundDevice();

	unsigned long computeFragmentSize(int sampleRate);
};
#endif
