// file menu for a user interface
// by Chris Kirmse

#include <string>
#include <vector>
#include <algorithm>

#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>

#include "primitives.h"

#include "menu.h"
#include "filemenu.h"

FileMenu::FileMenu()
{
	current_dir = ".";
	FindFiles();
}

FileMenu::~FileMenu()
{

}

int FileMenu::GetNumItems()
{
	return items.size();
}

const char * FileMenu::GetItemDisplayString(int index)
{
	return items[index].display_string.c_str();
}

void FileMenu::Selected(bool *was_file_selected)
{
	item_type item = items[GetSelectedIndex()];
	if (item.is_directory)
	{
		*was_file_selected = false;
		chdir(item.filename.c_str());
		FindFiles();
	}
	else
		*was_file_selected = true;

}

string FileMenu::GetSelectedFilename()
{
	return items[GetSelectedIndex()].filename;
}

void FileMenu::FindFiles()
{
	items.clear();

	DIR *dir = opendir(current_dir.c_str());

	struct dirent *dp;
	struct stat file_stat;

	while (1)
	{
		dp = readdir(dir);
		if (dp == NULL)
			break;

		item_type item;
		item.filename = string(dp->d_name);
		stat(dp->d_name,&file_stat);

		item.display_string = item.filename;
		if (S_ISDIR(file_stat.st_mode))
		{
			item.display_string.append("/");
			item.is_directory = true;
		}
		else
		{
			item.is_directory = false;
			// only .bin files please
			if (item.filename.size() < 4)
				continue;
			if (item.filename.compare(".bin",item.filename.size()-4,4) != 0 &&
				 item.filename.compare(".BIN",item.filename.size()-4,4) != 0)
			{
				continue;
			}


		}
		items.push_back(item);
	}
	closedir(dir);

	sort(items.begin(),items.end());
	SetSelectedIndex(0);
}

