// menu for a user interface
// by Chris Kirmse

#include "primitives.h"

#include "menu.h"

Menu::Menu()
{
	current_item_index = 0;

	x = 0;
	y = 0;
	width = 0;
	height = 0;

	row_height = 100;

	InitAlphaQuad(&background_quad,x,y,width,height,0,192,0,32);
	InitAlphaQuad(&current_item_quad,x,y,width,row_height,16,16,16,64);

	start_draw_index = 0;
	font = NULL;

	frames_down = 0;
	frames_up = 0;
}

Menu::~Menu()
{

}

void Menu::SetPosition(int new_x,int new_y)
{
	x = new_x;
	y = new_y;

	MoveAlphaQuad(&background_quad,x,y,width,height);
}
	
void Menu::SetSize(int new_width,int new_height)
{
	width = new_width;
	height = new_height;

	MoveAlphaQuad(&background_quad,x,y,width,height);
}

void Menu::Draw()
{
	DrawQuad(&background_quad);
	DrawQuad(&current_item_quad);

	int end_draw_index = start_draw_index + GetNumVisibleItems()-1;
	if (end_draw_index >= GetNumItems())
		end_draw_index = GetNumItems()-1;

	int draw_x = x;
	int draw_y = y;
	for (int i=start_draw_index;i<=end_draw_index;i++)
	{
		DrawString(GetItemDisplayString(i),draw_x,draw_y,font,true);
		draw_y += row_height;
	}
}

void Menu::SetFont(Font *new_font)
{
	font = new_font;

	row_height = font->h;

	SetCurrentItemQuad();
}

void Menu::Move(bool up,bool down)
{
	if (up && down)
	{
		frames_up = 0;
		frames_down = 0;
	}
	if (up == false)
		frames_up = 0;
	if (down == false)
		frames_down = 0;

	if (up)
	{
		frames_up++;
		if (frames_up == 1 || frames_up > 120 ||
			 (frames_up > FRAMES_INITIAL_DELAY && ((frames_up - FRAMES_INITIAL_DELAY) % FRAMES_REPEAT_DELAY) == 0))
		{
			MoveUp();
		}
	}

	if (down)
	{
		frames_down++;
		if (frames_down == 1 || frames_down > 120 ||
			 (frames_down > FRAMES_INITIAL_DELAY && ((frames_down - FRAMES_INITIAL_DELAY) % FRAMES_REPEAT_DELAY) == 0))
		{
			MoveDown();
		}
	}

}

int Menu::GetNumItems()
{
	return 0;
}

const char * Menu::GetItemDisplayString(int index)
{
	return "";
}

void Menu::SetSelectedIndex(int new_selected_index)
{
	if (current_item_index >= start_draw_index + GetNumVisibleItems())
		start_draw_index = current_item_index - GetNumVisibleItems()+1;

	current_item_index = new_selected_index;
	if (current_item_index < start_draw_index)
		start_draw_index = current_item_index;

	SetCurrentItemQuad();
}

int Menu::GetSelectedIndex()
{
	return current_item_index;
}

void Menu::MoveUp()
{
	current_item_index--;
	if (current_item_index < 0)
		current_item_index = 0;

	if (current_item_index < start_draw_index)
		start_draw_index = current_item_index;

	SetCurrentItemQuad();
}
	
void Menu::MoveDown()
{
	current_item_index++;
	if (current_item_index >= GetNumItems())
	{
		current_item_index = GetNumItems()-1;
		if (current_item_index < 0)
			current_item_index = 0;
	}

	if (current_item_index >= start_draw_index + GetNumVisibleItems())
		start_draw_index = current_item_index - GetNumVisibleItems()+1;

	SetCurrentItemQuad();
}

void Menu::SetCurrentItemQuad()
{
	MoveAlphaQuad(&current_item_quad,
					  x,y+(current_item_index-start_draw_index)*row_height,
					  width,row_height);

}

int Menu::GetNumVisibleItems()
{
	return height / row_height;
}
