// pause menu for a main ui
// by Chris Kirmse

#include <string>
#include <vector>

#include "Event.hxx"

#include "primitives.h"

#include "mainPS2.h"
#include "menu.h"
#include "pausemenu.h"

enum
{
	LEFT_DIFFICULTY = 0,
	RIGHT_DIFFICULTY = 1,
	COLOR_MODE = 2,
	BILINEAR_FILTER = 3,
	BLANK_1 = 4,
	QUIT = 5,
	NUM_STRINGS = 6,
};

PauseMenu::PauseMenu()
{
	for (int i=0;i<NUM_STRINGS;i++)
	{
		strings.push_back(string());
		SetString(i);
	}
}

PauseMenu::~PauseMenu()
{

}


int PauseMenu::GetNumItems()
{
	return strings.size();
}

const char * PauseMenu::GetItemDisplayString(int index)
{
	return strings[index].c_str();
}

void PauseMenu::Selected()
{
	switch (GetSelectedIndex())
	{
	case LEFT_DIFFICULTY :
	{
		Event *event = GetEvent();
		if (event->get(Event::ConsoleLeftDifficultyA))
		{
			event->set(Event::ConsoleLeftDifficultyA,0);
			event->set(Event::ConsoleLeftDifficultyB,1);
		}
		else
		{
			event->set(Event::ConsoleLeftDifficultyA,1);
			event->set(Event::ConsoleLeftDifficultyB,0);
		}
		SetString(GetSelectedIndex());
		break;
	}

	case RIGHT_DIFFICULTY :
	{
		Event *event = GetEvent();
		if (event->get(Event::ConsoleRightDifficultyA))
		{
			event->set(Event::ConsoleRightDifficultyA,0);
			event->set(Event::ConsoleRightDifficultyB,1);
		}
		else
		{
			event->set(Event::ConsoleRightDifficultyA,1);
			event->set(Event::ConsoleRightDifficultyB,0);
		}
		SetString(GetSelectedIndex());
		break;
	}

	case COLOR_MODE :
	{
		Event *event = GetEvent();
		if (event->get(Event::ConsoleBlackWhite))
		{
			event->set(Event::ConsoleColor,1);
			event->set(Event::ConsoleBlackWhite,0);
		}
		else
		{
			event->set(Event::ConsoleColor,0);
			event->set(Event::ConsoleBlackWhite,1);
		}
		SetString(GetSelectedIndex());
		break;
	}

	case BILINEAR_FILTER :
		SetFiltered(!GetFiltered());
		SetString(GetSelectedIndex());
		break;

	case QUIT :
		GoToUI();
		break;

	default :
		break;
	}
}

void PauseMenu::SetString(int index)
{
	string result = string();
	Event *event = GetEvent();

	switch (index)
	{
	case LEFT_DIFFICULTY :
		result.append("Left difficulty: ");
		if (event->get(Event::ConsoleLeftDifficultyA))
			result.append("A");
		else
			result.append("B");
		break;
	case RIGHT_DIFFICULTY :
		result.append("Right difficulty: ");
		if (event->get(Event::ConsoleRightDifficultyA))
			result.append("A");
		else
			result.append("B");
		break;
	case COLOR_MODE :
		result.append("Color mode: ");
		if (event->get(Event::ConsoleBlackWhite))
			result.append("black and white");
		else
			result.append("color");
		break;
	case BILINEAR_FILTER :
		result.append("Bilinear filter: ");
		if (GetFiltered())
			result.append("yes");
		else
			result.append("no");
		break;
	case QUIT :
		result.append("Quit");
		break;
	default :
		break;
	}
	strings[index] = result;
}
