#ifndef PRIMITIVES_H
#define PRIMITIVES_H

#include "ps2gs.h"
#include "ps2dma.h"
#include "packet.h"
#include "texture.h"

// the regdata is aligned to 128 bit boundary, so it can use extra space
typedef struct
{
	GifPacket packet;
	int x,y,packetsize;
	u64 regdata[5 + 2/*alignment issues*/];
} Pixel;

typedef struct
{
	GifPacket packet;
	int x0,y0,x1,y1,packetsize;
	u64 regdata[50 + 2/*alignment issues*/];
} Line;

typedef struct
{
	GifPacket packet;
	int x,y,packetsize;
	u64 regdata[25 + 2/*alignment issues*/];
} Triangle;

typedef struct
{
	GifPacket packet;
	int x,y,w,h,packetsize;
	u64 regdata[14 + 2/*alignment issues*/];
} Quad;

typedef struct
{
	GifPacket packet;
	Texture *tex;
	int x,y,w,h,packetsize;
	u64 regdata[20 + 2/*alignment issues*/];
} QuadTex;

typedef struct
{
	GifPacket packet;
	Texture *tex;
	int x,y,w,h,space;
	u64 regdata[400 + 2/*alignment issues*/];
} Font;
	
	
void InitPixel(Pixel *p, int x, int y, int r, int g, int b);
void DrawPixel(Pixel *p);

void InitLine(Line *l, int x0, int y0, int x1, int y1, int r, int g, int b);
void InitLineStrip(Line *l, int x, int y, int w, int h, int *verts, int numverts, int r, int g, int b);
void DrawLine(Line *l);

void InitTriangle(Triangle *t, int x, int y, int v0x, int v0y, int v1x, int v1y,
			      int v2x, int v2y, int r, int g, int b);
void InitShadedTriangle(Triangle *t, int x, int y, int v0x, int v0y, int v1x, int v1y,
			            int v2x, int v2y, int r0, int g0, int b0, int r1, int g1, 
			            int b1, int r2, int g2, int b2);
void InitTriangleStrip(Triangle *t, int x, int y, int v0x, int v0y, int v1x, int v1y,
			           int v2x, int v2y, int *verts, int numverts, int r, int g, int b);
void InitTriangleFan(Triangle *t, int x, int y, int v0x, int v0y, int v1x, int v1y,
			           int v2x, int v2y, int *verts, int numverts, int r, int g, int b);			           
void DrawTriangle(Triangle *t);

void InitAlphaQuad(Quad *q, int x, int y, int w, int h, int r, int g, int b, int a);
void MoveAlphaQuad(Quad *q,int x,int y,int w,int h);
void InitQuad(Quad *q, int x, int y, int w, int h, int r, int g, int b);
void MoveQuad(Quad *q,int x,int y,int w,int h);
void DrawQuad(Quad *q);

void InitQuadTex(QuadTex *q, int x, int y, int w, int h, int trans, int filter, Texture *t);
void SetQuadTexFilter(QuadTex *q,int filter);
void MoveQuadTex(QuadTex *q,int x,int y,int w,int h);
void SetQuadTexUV(QuadTex *q,int uvx0,int uvy0,int uvx1,int uvy1);
void DrawQuadTex(QuadTex *q);

void InitFont(Font *f, Texture *t, int spacing);
void DrawString(const char *str, int x, int y, Font *f,bool trans);

#endif
