#ifndef TEXTURE_H
#define TEXTURE_H

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <memory.h>
#include <malloc.h>
#include <fcntl.h>
#include <math.h>
#include <unistd.h>
#include "ps2gs.h"

typedef unsigned char  		Byte;
typedef unsigned short 		Word;
typedef unsigned long  		Dword;
typedef long		   		Long;
typedef unsigned long long	Qword;

#define BITMAP_ID      0x4D42

typedef struct
{
	Word    bfType;
    Dword   bfSize;
    Word    bfReserved1;
    Word    bfReserved2;
    Dword   bfOffBits;
	
} BitmapFileHeader;

typedef struct
{
	Dword	biSize;
	Long	biWidth;
	Long	biHeight;
	Word	biPlanes;
	Word	biBitCount;
	Dword	biCompression;
	Dword	biSizeImage;
	Long	biXPelsPerMeter;
	Long	biYPelsPerMeter;
	Dword	biClrUsed;
	Dword	biClrImportant;
	
} BitmapInfoHeader;

typedef struct
{
	BitmapFileHeader	bitmapfileheader;
	BitmapInfoHeader	bitmapinfoheader;
} BitmapFile;

class Texture
{
public:
	
	Texture();
	~Texture();

	bool Create8BitPalettized(int width,int height);
	void SetPaletteEntry(Byte index,Byte r,Byte g,Byte b,Byte a);

	int LoadBitmap(char *filename);

	void ResendPaletteToGS();
	void ResendImageToGS();

	int w,h;					//width,height
	int log2width,log2height;	//width,height in texels
	int image_adr;				//address of image data in GSRAM
	int psm;					//pixel storage format
	int clut_adr;				//address of CLUT in GSRAM
	int clut_psm;				//CLUT pixel storage format
	int clut_load;				//CLUT buffer load control 
	
	Byte* image_data;			//image data in main memory
	Byte* clut_data;			//CLUT data in main memory

private:
	
};

//Helper functions
int FlipBitmap(Byte *image, int bytes_per_line, int height);

void CaptureScreen(char *filename);

Word Log2(Word n);

unsigned int swapBits3and4(unsigned int value);

#endif
